package com.sharix.sportsmanfriend.view.fragments;

import android.Manifest;
import android.content.pm.PackageManager;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import com.google.android.gms.location.FusedLocationProviderClient;
import com.google.android.gms.location.LocationServices;
import com.sharix.sportsmanfriend.BaseFragmentClass;
import com.sharix.sportsmanfriend.R;
import org.osmdroid.api.IMapController;
import org.osmdroid.tileprovider.tilesource.TileSourceFactory;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapView;

public class MapFragment extends BaseFragmentClass {
    // константы
    private final int REQUEST_PERMISSIONS_REQUEST_CODE = 1;
    private final int PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION = 2;
    private final String KEY_ZOOM_LEVEL = "zoomLevel";
    private final String KEY_CENTER_LATITUDE = "centerLatitude";
    private final String KEY_CENTER_LONGITUDE = "centerLongitude";
    //Элементы
    private MapView map = null;
    private ImageButton btnZoomIn, btnZoomOut, btnMyLocation;
    private LinearLayout btnOpenEventList;
    //определяет последнюю известную локацию
    private FusedLocationProviderClient fusedLocationProviderClient;
    //контроллер карты OSM
    IMapController mapController;

    private double latitude, longitude;

    private static final int REQUEST_CODE = 1;
    private String[] permissions = {Manifest.permission.READ_CONTACTS};

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

        View view = inflater.inflate(R.layout.fragment_map, container, false);

        //
        btnZoomIn = view.findViewById(R.id.btn_zoomIn);
        btnZoomOut = view.findViewById(R.id.btn_zoomOut);
        btnMyLocation = view.findViewById(R.id.btnMyLocation);
        btnOpenEventList = view.findViewById(R.id.btnOpenEventList);

        map = view.findViewById(R.id.map);
        mapController = map.getController();
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(getActivity());

        if (!hasSavedState()){
            configureMap(55.934473, 37.497081, 15);
        }

        ActivityCompat.requestPermissions(getActivity(), permissions, REQUEST_CODE);
        askPermissions();
        addClickEventToAllButtons();

        return view;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case REQUEST_PERMISSIONS_REQUEST_CODE: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    getActivity().recreate();
                }
            }
            case PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    getActivity().recreate();
                }
            }
        }
    }

    private void configureMap(double Latitude, double Longitude, int Zoom) {
        map.setTileSource(TileSourceFactory.DEFAULT_TILE_SOURCE);
        map.setMultiTouchControls(true);
        map.setMaxZoomLevel(18.0);

        mapController.setZoom(Zoom);
        GeoPoint startPoint = new GeoPoint(Latitude, Longitude);
        mapController.setCenter(startPoint);

        map.setBuiltInZoomControls(false);
    }

    private void askPermissions() {
        if (ContextCompat.checkSelfPermission(getContext(), Manifest.permission.INTERNET) != PackageManager.PERMISSION_GRANTED ||
            ContextCompat.checkSelfPermission(getContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED ||
            ContextCompat.checkSelfPermission(getContext(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
            String[] permission = {Manifest.permission.INTERNET, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.ACCESS_FINE_LOCATION};
            requestPermissions(permission, REQUEST_PERMISSIONS_REQUEST_CODE);
        }
    }

    private void addClickEventToAllButtons() {

        btnZoomIn.setOnClickListener(v -> {
            if (map.canZoomIn()) {
                mapController.zoomIn();
            }
        });

        btnZoomOut.setOnClickListener(v -> {
            if (map.canZoomOut()) {
                mapController.zoomOut();
            }
        });

        btnMyLocation.setOnClickListener(v -> {
            if (ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED) {
                String[] permission = {Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION};
                requestPermissions(permission, PERMISSIONS_REQUEST_ACCESS_FINE_LOCATION);
            }

            fusedLocationProviderClient.getLastLocation()
                    .addOnSuccessListener(getActivity(), location -> {
                        if (location != null) {
                            latitude = location.getLatitude();
                            longitude = location.getLongitude();
                            mapController.setCenter(new GeoPoint(latitude, longitude));
                            mapController.setZoom(18);
                        }
                    });

        });

        btnOpenEventList.setOnClickListener(v -> {
            // Получаем менеджер фрагментов
            FragmentManager fragmentManager = getActivity().getSupportFragmentManager();

            // Создаем транзакцию фрагментов
            FragmentTransaction fragmentTransaction = fragmentManager.beginTransaction();

            // Создаем новый фрагмент, который мы хотим открыть
            EventListFragment myFragment = new EventListFragment();

            // Заменяем текущий фрагмент на новый
            fragmentTransaction.replace(R.id.frameWindow, myFragment);

            // Добавляем транзакцию в стек возврата, чтобы пользователь мог вернуться назад
            fragmentTransaction.addToBackStack(null);

            // Применяем транзакцию
            fragmentTransaction.commit();
        });
    }

    @Override
    protected boolean hasSavedState() {
        Bundle state = getSavedState();

        if (state == null) {
            return false;
        }

        int zoomLevel = state.getInt(KEY_ZOOM_LEVEL);
        double centerLatitude = state.getDouble(KEY_CENTER_LATITUDE);
        double centerLongitude = state.getDouble(KEY_CENTER_LONGITUDE);
        configureMap(centerLatitude, centerLongitude, zoomLevel);

        return true;
    }

    @Override
    protected Bundle getStateToSave() {
        Bundle state = new Bundle();

        state.putInt(KEY_ZOOM_LEVEL, map.getZoomLevel());
        state.putDouble(KEY_CENTER_LATITUDE, map.getMapCenter().getLatitude());
        state.putDouble(KEY_CENTER_LONGITUDE, map.getMapCenter().getLongitude());

        return state;
    }
}